/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is zinc
 *
 * The Initial Developer of the Original Code is UniServices Ltd
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): UniServices Ltd, Andrew Miller
 *                 Carey Stevens, Shane Blackett
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function RunnableFunctionWrapper(aFunction)
{
  this.run = aFunction;
}

RunnableFunctionWrapper.prototype =
{
  QueryInterface: function(aIID)
  {
    if (aIID.equals(Components.interfaces.nsIRunnable) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;
    throw Components.results.NS_NOINTERFACE;
  }
};

function sceneReady(aFunction)
{
  this.run = aFunction;
}

sceneReady.prototype =
{
  QueryInterface: function(aIID)
  {
    if (
        aIID.equals(Components.interfaces.CmguiISceneViewerResultMonitor)
       )
    {
      return this;
    }
    throw Components.results.NS_NOINTERFACE;
  },

  sceneViewerReady: function(sceneViewer)
  {
	  this.run(sceneViewer);
  },

  sceneViewerFailed: function()
  {
	  throw Components.results.NS_UNEXPECTED;
  }
};

function zincDefineMemoryBlock(dm, contentURL, memoryName)
{

  if ((contentURL.indexOf("chrome://") != -1)
  || (contentURL.indexOf("http://") != -1)
  || (contentURL.indexOf("https://") != -1)
  || (contentURL.indexOf("ftp://") != -1)
  || (contentURL.indexOf("jar:") != -1)
  || (contentURL.indexOf("file://") != -1)) // contains protocol, so assume to be an absolute url already
  {
    var absoluteURL = contentURL;
  }
  else
  {

    // make an absolute url using current directory as the base (not using
    // href to ignore search and hash extensions)

    // we cannot assemble from protocol, host and path as these variables 
	 // are not correctly populate if the url has a jar prefix
	 // instead evaluate the window.location in string context and strip off search and hash extensions
    //var url = window.location.protocol + "//" + window.location.host + window.location.pathname;
	 
	  // what about window.location.href?
    var url = window.location + "";

	 // strip off hash and any following characters
	 var hashIndex = url.indexOf('#');	 
	 if (hashIndex != -1) {
		 url = url.substring(0, hashIndex);
	 }

	 // strip off question mark and any following characters
	 var questionIndex = url.indexOf('?');
	 if (questionIndex != -1) {
		 url = url.substring(0, questionIndex);
	 }
	
	 // srip off path to get absolute URL
	 var absoluteDirectoryURL = url.substring(0, (url.lastIndexOf("/")+1));

	 var absoluteURL = absoluteDirectoryURL + contentURL;
  }
 
  try
  {
	  //	 alert("Absolute URL is:" + absoluteURL);
    dm.defineMemoryBlockFromUrl(absoluteURL, memoryName);
  } catch(ex) {
	  alert("Error downloading from absoluteURL:" + absoluteURL + "\nException is " +ex);
  }
}


function zincVersion()
{
  return Components.interfaces.zincICmgui.MAJOR_VERSION + '.' + Components.interfaces.zincICmgui.MINOR_VERSION  + '.' + Components.interfaces.zincICmgui.SUBMINOR_VERSION  + '.' + Components.interfaces.zincICmgui.BUILD_VERSION
}


function zincCheckValidVersion(minVersion, maxVersion)
{
  var min_version_array = minVersion.split(".");
  var max_version_array = maxVersion.split(".");

  var valid = true

  if (Components.interfaces.zincICmgui.MAJOR_VERSION 
	  < min_version_array[0])
  {
	  valid = false;
  }
  else if (Components.interfaces.zincICmgui.MAJOR_VERSION 
	  == min_version_array[0])
  {
	  if ((min_version_array.length > 1) && (min_version_array[1] != "*"))
	  {
		  if (Components.interfaces.zincICmgui.MINOR_VERSION 
			  < min_version_array[1])
		  {
			  valid = false;
		  } 
		  else if (Components.interfaces.zincICmgui.MINOR_VERSION 
			  == min_version_array[1])
		  {
			  if ((min_version_array.length > 2) && (min_version_array[2] != "*"))
			  {
				  if (Components.interfaces.zincICmgui.SUBMINOR_VERSION 
					  < min_version_array[2])
				  {
					  valid = false;
				  } 
				  else if (Components.interfaces.zincICmgui.SUBMINOR_VERSION 
					  == min_version_array[2])
				  {
					  if ((min_version_array.length > 3) && (min_version_array[3] != "*") &&
						  (Components.interfaces.zincICmgui.BUILD_VERSION 
							  < min_version_array[3]))
					  {
						  valid = false;
					  }
				  }
			  }
		  }
	  }
  }

  if (Components.interfaces.zincICmgui.MAJOR_VERSION 
	  > max_version_array[0])
  {
	  valid = false;
  }
  else if (Components.interfaces.zincICmgui.MAJOR_VERSION 
	  == max_version_array[0])
  {
	  if ((max_version_array.length > 1) && (max_version_array[1] != "*"))
	  {
		  if (Components.interfaces.zincICmgui.MINOR_VERSION 
			  > max_version_array[1])
		  {
			  valid = false;
		  } 
		  else if (Components.interfaces.zincICmgui.MINOR_VERSION 
			  == max_version_array[1])
		  {
			  if ((max_version_array.length > 2) && (max_version_array[2] != "*"))
			  {
				  if (Components.interfaces.zincICmgui.SUBMINOR_VERSION 
					  > max_version_array[2])
				  {
					  valid = false;
				  } 
				  else if (Components.interfaces.zincICmgui.SUBMINOR_VERSION 
					  == max_version_array[2])
				  {
					  if ((max_version_array.length > 3) && (max_version_array[3] != "*") &&
						  (Components.interfaces.zincICmgui.BUILD_VERSION 
							  > max_version_array[3]))
					  {
						  valid = false;
					  }
				  }
			  }
		  }
	  }
  }

  return (valid);
}


function zincInitialise(CmguiReadyFunction)
{
	var mcg = Components.classes["@physiomeproject.org/zinc;1"]
		.createInstance(Components.interfaces.zincICmgui);

	window.mcg = mcg;

	//Is this a runnable object or a function?
	if (undefined != CmguiReadyFunction.cmguiReady)
	{
		CmguiReadyFunction.cmguiReady(mcg.createCmguiCommandData());
	}
	else
	{
		CmguiReadyFunction(mcg.createCmguiCommandData());
	}
};

function zincCreateSceneViewer(plugin, CommandDataObject, SceneReadyFunction)
{
	var sceneReadyObject;
	 //Is this a runnable object already?
	 if (undefined != SceneReadyFunction.sceneViewerReady)
	 {
		 sceneReadyObject = SceneReadyFunction;
	 }
	 else
	 {
		 sceneReadyObject = new sceneReady(SceneReadyFunction);
	 }

	CommandDataObject.createSceneViewer(
		plugin.QueryInterface(Components.interfaces.zincIPlugin), 
		sceneReadyObject);
}

function zincCreateDownloadMonitor(CommandDataObject, DownloadsReadyFunction)
{
    var dm = Components.classes["@physiomeproject.org/zinc/zincDownloadMonitor;1"]
                      .createInstance(Components.interfaces.zincIDownloadMonitor);
    dm.commandData = CommandDataObject;

	 //Is this a runnable object already?
	 if (undefined != DownloadsReadyFunction.run)
	 {
		 dm.onDownloadsCompleted = DownloadsReadyFunction;
	 }
	 else
	 {
		 dm.onDownloadsCompleted = new RunnableFunctionWrapper(DownloadsReadyFunction);
	 }

	 return (dm);
}

function zincGoToURLAndClose(url) {
  window.open(url);
  window.close();
}

function zincGoToURL(url) {
  window.open(url);
}
